@@ -30,17 +30,20 @@
 #include "slave.h"
 #include "verify.h"
 
-static const gchar RCSid[]="$Id: verify-crypt.c,v 1.4 2001/01/31 01:16:44 jirka Exp $";
+static const gchar RCSid[]="$Id: verify-crypt.c,v 1.10 2001/05/13 06:44:29 jirka Exp $";
 
 
 /* Configuration option variables */
 extern gboolean GdmVerboseAuth;
 extern gboolean GdmAllowRoot;
+extern gboolean GdmAllowRemoteRoot;
 
 
 /**
  * gdm_verify_user:
+ * @username: Name of user or NULL if we should ask
  * @display: Name of display to register with the authentication system
+ * @local: boolean if local
  *
  * Provides a communication layer between the operating system's
  * authentication functions and the gdmgreeter. 
@@ -49,45 +52,103 @@ extern gboolean GdmAllowRoot;
  */
 
 gchar *
-gdm_verify_user (const gchar *display) 
+gdm_verify_user (const char *username,
+		 const gchar *display,
+		 gboolean local) 
 {
     gchar *login, *passwd, *ppasswd;
     struct passwd *pwent;
 
+    if (local)
+	    gdm_slave_greeter_ctl_no_ret (GDM_STARTTIMER, "");
+
     /* Ask for the user's login */
-    login = gdm_slave_greeter_ctl (GDM_PROMPT, _("Login:"));
+    if (username == NULL) {
+	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Login:"));
+	    if (login == NULL ||
+		gdm_slave_greeter_check_interruption (login)) {
+		    if (local)
+			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
+		    g_free (login);
+		    return NULL;
+	    }
+    } else {
+	    login = g_strdup (username);
+    }
+
     pwent = getpwnam (login);
         
     ppasswd = !pwent ? NULL : pwent->pw_passwd;
     
     /* Request the user's password */
-    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
+    if (gdm_string_empty (ppasswd)) {
+	    /* eeek a passwordless account */
+	    passwd = g_strdup ("");
+    } else {
+	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
+	    if (passwd == NULL)
+		    passwd = g_strdup ("");
+	    if (gdm_slave_greeter_check_interruption (passwd)) {
+		    if (local)
+			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
+		    g_free (login);
+		    g_free (passwd);
+		    return NULL;
+    }
 
-    /* If verbose authentication is enabled, output messages from the
-     * authentication system */
-    if (GdmVerboseAuth) {
+    }
 
-	if (!pwent) {
+    if (local)
+	    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
+
+    if (pwent == NULL) {
 	    gdm_error (_("Couldn't authenticate %s"), login);
-	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Login incorrect"));
+	    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR, _("Login incorrect"));
+	    g_free (login);
+	    g_free (passwd);
 	    return NULL;
-	}
-    
-	if (GdmAllowRoot == 0 && pwent->pw_uid == 0) {
-	    gdm_error (_("Root login disallowed on display '%s'"), display);
-	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Root login disallowed"));
-	    return NULL;
-	}	
     }
 
     /* Check whether password is valid */
-    if (!passwd || !ppasswd || strcmp (crypt (passwd, ppasswd), ppasswd)) {
-
-	if (GdmVerboseAuth)
-	    gdm_slave_greeter_ctl (GDM_MSGERR, _("Login incorrect"));
-	
-	return NULL;
+    if (ppasswd == NULL || (ppasswd[0] != '\0' &&
+			    strcmp (crypt (passwd, ppasswd), ppasswd) != 0)) {
+	    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR, _("Login incorrect"));
+	    g_free (login);
+	    g_free (passwd);
+	    return NULL;
     }
+
+    if ( ( ! GdmAllowRoot ||
+	  ( ! GdmAllowRemoteRoot && ! local) ) &&
+	pwent->pw_uid == 0) {
+	    gdm_error (_("Root login disallowed on display '%s'"), display);
+	    if (GdmVerboseAuth) {
+		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+						  _("Root login disallowed"));
+	    } else {
+		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+						  _("Login incorrect"));
+	    }
+	    g_free (login);
+	    g_free (passwd);
+	    return NULL;
+    }
+
+    /* check for the standard method of disallowing users */
+    if (pwent->pw_shell != NULL &&
+	strcmp (pwent->pw_shell, "/bin/false") == 0) {
+	    gdm_error (_("User %s not allowed to log in"), login);
+	    if (GdmVerboseAuth) {
+		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+						  _("Login disabled"));
+	    } else {
+		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+						  _("Login incorrect"));
+	    }
+	    return NULL;
+    }	
+
+    g_free (passwd);
     
     return login;
 }
@@ -110,10 +171,11 @@ gdm_verify_setup_user (const gchar *login, const gchar *display)
 /**
  * gdm_verify_cleanup:
  *
- * Unregister the user's session */
+ * Unregister the user's session
+ */
 
 void
-gdm_verify_check (void)
+gdm_verify_cleanup (void)
 {
 }
 
@@ -130,4 +192,10 @@ gdm_verify_check (void)
 {
 }
 
+/* used in pam */
+void
+gdm_verify_env_setup (void)
+{
+}
+
 /* EOF */
